#ifndef LEDOutput_cpp
#define LEDOutput_cpp

#include <Arduino.h>

#define PWMRANGE 1023
#define FADE_DURATION 5000.0f

class LEDOutput {
	unsigned int _pin;
	bool _invert;
	unsigned long _fadeStart;
	float _targetBrightness, _startBrightness, _currentBrightness;

	float getOutputMultiplier() {
		float elapsed = (millis() - _fadeStart) / FADE_DURATION;

		if (elapsed >= 1)
			return _targetBrightness;

		_currentBrightness = (_targetBrightness - _startBrightness) * elapsed + _startBrightness;
		return _currentBrightness;
	}

	void output(float multiplier) {
		auto value = multiplier * PWMRANGE;
		if (_invert)
			value = PWMRANGE - value;
		analogWrite(_pin, value);
	}

public:
	LEDOutput(unsigned int pin, bool invert = false) {
		_pin = pin;
		_invert = invert;

		_fadeStart = millis();
		_startBrightness = _currentBrightness = _targetBrightness = 0;

		pinMode(_pin, OUTPUT);
	}

	void writeFraction(float fraction, bool immediate = false){
		if (fraction > 1){
			Serial.print("Fraction exceeds 1: ");
			Serial.println(fraction);
			return;
		}

		if (immediate) {
			_startBrightness = fraction;
			_targetBrightness = fraction;
			output(fraction);
			return;
		}
		
		if (fraction == _targetBrightness)
			return;

		_fadeStart = millis();
		_startBrightness = _currentBrightness;
		_targetBrightness = fraction;
	}

	void loop() {
		output(getOutputMultiplier());
	}
};

#endif
