#ifndef Lighting_h
#define Lighting_h

#include <FastLED.h>
#include "NaturalLight.h"
#include "Weather.h"
#include "LEDOutput.cpp"

DECLARE_GRADIENT_PALETTE(_sunrise_p);

class Lighting {
	int _ledCount;
	LEDOutput* _whiteLed;
	LEDOutput* _redLed;
	LEDOutput* _greenLed;
	LEDOutput* _blueLed;
	NaturalLight *_naturalLight;
	Weather* _weather;
	float _cloudCoverLimit;
	CRGB* _leds;
	CRGBPalette16 _sunrise = _sunrise_p;
	unsigned long _nextLightningFlash = 0;
	float _heatIndex;
	float _brightness;
	byte _lastHeatIndex, _lastBrightness;
	bool _useStrip, _usePWM;

	float _currentHeatIndex();
	float getIndexMultiplier(int now, int startTime, int endTime, bool swap);
	float getHeatIndex(int time);
	float getPaletteHeatIndex(int time);
	float getBrightness();
	float getPaletteBrightness();
	float byteToFraction(byte input);
	void outputColour(CRGB colour);
	void updateRGB(int now);
	void updateWhite();

  public:
	Lighting(NaturalLight* _naturalLight, Weather* _weather, float cloudCoverLimit);

	template<uint8_t dataPin, int ledCount, LEDOutput* whiteLed> void setupStrip() {
		_ledCount = ledCount;
		FastLED.addLeds<WS2811, dataPin, GRB>(_leds, _ledCount).setCorrection(TypicalSMD5050).setTemperature(Tungsten40W);
		_whiteLed = whiteLed;
		_whiteLed->writeFraction(0);
		_useStrip = true;
	}

	template<LEDOutput*  redLed, LEDOutput* greenLed, LEDOutput* blueLed, LEDOutput* whiteLed> void setupPWM() {
		_redLed = redLed;
		_blueLed = blueLed;
		_greenLed = greenLed;
		_whiteLed = whiteLed;
		_redLed->writeFraction(0, true);
		_greenLed->writeFraction(0, true);
		_blueLed->writeFraction(0, true);
		_whiteLed->writeFraction(0, true);
		_usePWM = true;
	}

	void update(int time);
	float getCurrentHeatIndex() { return _heatIndex; }
	float getCurrentBrightness() { return _brightness; }
};

#endif