#include "NaturalLight.h"
#include <ESP8266HTTPClient.h>
#include <ArduinoJson.h>
#include <time.h>
#include <Arduino.h>

NaturalLight::NaturalLight(WiFiClient wifiClient, const char* lat, const char* lon, int timezoneOffset) {
	_wifiClient = wifiClient;
	_timezoneOffset = timezoneOffset;
	sprintf(_url, SOLAR_EVENT_URL, lat, lon);
}

String NaturalLight::getSolarEventJson() {
	HTTPClient http;
	Serial.print("Making request: ");
	Serial.println(_url);
	http.begin(_wifiClient, _url);
	int httpCode = http.GET();
	String result = "";
	if (httpCode > 0)
		result = http.getString();
	http.end();

	Serial.print("Result: ");
	Serial.println(result);
	return result;
}

int NaturalLight::getEpoch(const char *str) {
	struct tm time;
	int year, month;
	sscanf(str, "%d-%d-%dT%d:%d:%d+00:00", &year, &month, &time.tm_mday, &time.tm_hour, &time.tm_min, &time.tm_sec);
	time.tm_year = year - 1900;
	time.tm_mon = month - 1;
	return mktime(&time);
}

int NaturalLight::adjustForTimezone(int epoch) {
	return epoch + _timezoneOffset;
}

void NaturalLight::update() {
	String json = getSolarEventJson();
	StaticJsonDocument<1024> document;

	DeserializationError error = deserializeJson(document, json);
	if (error)
		return;

	const char *astronomicalTwilightBegin = document["results"]["astronomical_twilight_begin"];
	const char *sunrise = document["results"]["sunrise"];
	const char *sunset = document["results"]["sunset"];
	const char *astronomicalTwilightEnd = document["results"]["astronomical_twilight_end"];

	_astronomicalTwilightBegin = adjustForTimezone(getEpoch(astronomicalTwilightBegin));
	_sunrise = adjustForTimezone(getEpoch(sunrise));
	_sunset = adjustForTimezone(getEpoch(sunset));
	_astronomicalTwilightEnd = adjustForTimezone(getEpoch(astronomicalTwilightEnd));
}